;============================================================================
; (c) Copyright Elect Software International Inc., 1993, Toronto. Anyone can
; use this code for anything as long as it is not resold as a software
; development resource, as long as the copyright notice isn't removed, as
; long as changes are clearly marked as to authorship, and as long as users
; indemnify Elect from any liability.
; Comments welcome. Henrik Bechmann, CIS:72701,3717; Tel:416-534-8176.
;============================================================================

; ButtonMan Version 1.5, January, 1993

;==============================================================================
;                       ButtonMan DESCRIPTION
;==============================================================================
;
; ButtonMan allows the creation of any number of concurrent buttonbars on
; screen. These button bars can be independently configured to be horizontally
; or vertically oriented, and the buttons can be radio buttons, toggle buttons,
; or checkbox buttons. Many other configuration options are available,
; inlcuding color.
;
; The developer can associate a procedure to be fired with each button, or not.
;
; End users can reconfigure their own buttonbars, and save their configs
; in their private directories. They can change orientation, button styles,
; and a variety of window attributes, as well as all colors of their buttonbars.
; All buttonbars can be independently and separately configured, regardless
; of how many concurrent buttonbars exist.
;
; ButtonMan Version 1.5 requires the presence of EventMan version 1.1, and
; PackMan version 1. If ButtonMan.UserCanConfigure = True then ColorPal
; version 1 is required as well.
;
; ButtonMan version 1.5 is compatible with version 1 except
; that PropertyBag["ButtonColor"] is now used for button coloring where
; PropertyBag["CanvasColor"] was used in version 1.
;
; For hotkey support, if you specify a LabelSpec for a button with a hotkey
; delimited by tildes (eg. "My~H~otKey"), then a hotkey will automatically
; be created and registered with EventMan. ButtonMan assumes that Hotkeys are
; unique no matter how many buttonbars are present. Registration with EventMan
; is done at the default level (ie. no ObjectTag specified). Hot keys are
; interpreted as Alt-Char combinations, where Char is 0-9 or A-Z. Hotkeys
; are case insensitive.
;
; For mouse support, ButtonMan automatically registers all mouse buttons with
; EventMan for the ButtonBarTag (HandlerBag["ObjectTag"] = ButtonBarTag).
; ButtonMan assumes that EventMan is configured for
; "EventMan.SetIndexedObjectTag" or higher.
;
; For the Floating attribute, note that PropertyBag["MustFloat"] must be
; set to False (the default is True) if WindowBag["Floating"] = False is to
; take effect. The reason is to prevent the end user from changing a floating
; canvas to non-floating (using the buttonbar property inspector) in
; situations where the change could cause some grief for the application.
;
; See ButtonDem.sc and Keybar.sc for some examples or how to use ButtonMan.
;
;==============================================================================
;                       ButtonMan INTERFACE
;==============================================================================
; MESSAGES:
; ---------
; ButtonMan.Constructor()
; ButtonMan.Destructor()
; ButtonMan.MakeButtonBar(PropertyBag,ButtonBag,WindowBag,ColorBag)
;    Creates a ButtonBar, and registers mouse events for the button bar with
;       EventMan.
;    PropertyBag[] must contain an "ObjectTag" cell with the name of the
;       buttonbar
;    PropertyBag[] may contain any of the following (shown here with defualts)
;       PropertyBag["Orientation"] = "Horizontal" ; or "Vertical"
;       PropertyBag["Action"] = "Radio" ; or "Toggle" or "Checkbox"
;       PropertyBag["ButtonStyle"] = "Border" ; or "Shadow" or "Flat"
;       PropertyBag["UserScript"] = "" ; specify eg. "!MyBar", no path, no ext.
;       PropertyBag["WrapCount"] = 5 ; number of buttons on one row or column
;       PropertyBag["CanvasColor"] = 96 + 10 ; Background for the text
;       PropertyBag["ButtonColor"] = 96 + 10 ; Background for the text
;       PropertyBag["HighlightColor"] = 96 + 15 ; for the border
;       PropertyBag["ShadowColor"] = 96 + 0 ; for the border
;       PropertyBag["SelectedColor"] = 96 + 14 ; for the selected button text
;       PropertyBag["HotKeyColor"] = 80 + 12 ; for hotkeys, if any
;       PropertyBag["SelectedHotKeyColor"] = 96 + 12 ; for hotkeys, if any
;       PropertyBag["DisableColor"] = 96 + 7 ; for disabled buttons
;       PropertyBag["MustFloat"] = True
;
;    ButtonBag[] must have a list of the buttons in the following format:
;       ButtonBag[ButtonTag] = Chr(ButtonSequence) [+ ProcName]
;                        [+ ",<LabelSpec>"] ; note the required comma
;           where ButtonSequence numbers must be in sequence from 1 to the
;           total number of buttons, and ProcName is optional. ProcName is
;           fired when the button is pressed (on the mouse UP action).
;           LabelSpec in format like "This~B~utton". LabelSpec is required
;           for mixed case presentation. Tilde (~) delimiters for a letter
;           create a hotkey for the button.
;    WindowBag and ColorBag can contain any valid cells recognized by
;       Paradox. For WindowBag use (shown with defaults in parentheses)
;          Title
;          OriginRow
;          OriginCol
;          CanMove (True)
;          Floating (True)
;          HasShadow (True)
;          HasFrame (True)
;          CanResize (False)
;    For ColorBag set any valid color attribute. Any Color Attribute (0 - 4)
;       not set is set to PropertyBag["ShadowColor"]
; ButtonMan.EnableButton(ButtonBarTag,ButtonTag,Boolean) ; Boolean = False sets
;                       ; color to DisableColor and disables button.
;                       ; return true if it makes setting, false otherwise
; ButtonMan.SelectButton(ButtonBarTag,ButtonTag) ; pushes button, sets
;                       ; ButtonMan.MethodToFire if button was not set, and
;                       ; if there is a method (proc) associated with the
;                       ; button.
; ButtonMan.ResetButton(ButtonBarTag) ; resets current "Toggle" type button
; ButtonMan.DestroyButtonBar(ButtonBarTag)
; The following are automatically registered to EventMan by ButtonMan:
; ButtonMan.HandleMouse() ; default mouse handling procedure
; ButtonMan.HandleHotKeys() ; default hotkey handling procedure
;
; PROPERTIES:
; -----------
; ButtonMan.Property[ButtonBarTag + "ButtonTag"] =
;        <last selected button tag> ; readonly
; ButtonMan.Property[ButtonBarTag + "ButtonNumber"] = ""
;        ; Chr(ButtonNumber), readonly, last selected button
; ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] = Boolean ; readonly
;    Useful for checkboxes to determine which buttons are down.
; ButtonMan.MethodToFire ; set by ButtonMan.SelectButton according to selected
;                        ; button if button had not been previously selected.
;                        ; Fired on mouse UP by ButtonMan.HandleMouse(),
;                        ; and by HandleBarHotKey() and HandleAnyHotKey() if
;                        ; found to be non-blank.
; ButtonMan.ButtonBarTag ; The last buttonbar to have received a
;                        ; ButtonMan.SelectButton() call, whether internal
;                        ; (from handlemouse or handleHotKeys) or external
;                        ; (direct call from application).
; ButtonMan.MethodBag[]  ; contains default mouse event handlers for all
;                        ; buttons.
; ButtonMan.HotKeyMethod ; The proc used to invoke all hotkeys, default
;                        ; "ButtonMan.HandleHotKeys"
; ButtonMan.UserCanConfigure ; True (default) or false for property inspector
; ButtonMan.ShadowStyleOffset ; 0 default, determines whether there is a blank
;                             ; row above the first row of horizontal shadow
;                             ; style buttons
;==============================================================================
;                       ButtonMan IMPLEMENTATION
;==============================================================================

Proc ButtonMan.Constructor()
   Private
      SysInfoBag

   ;--------------------------------------------------------------------------
   ; Current screen dimensions...
   ;--------------------------------------------------------------------------
   Sysinfo To SysInfoBag

   ;--------------------------------------------------------------------------
   ; Default ButtonBar window attributes...
   ;--------------------------------------------------------------------------
   Dynarray ButtonMan.DefaultWindowBag[]
   ButtonMan.DefaultWindowBag["Floating"] = True
   ButtonMan.DefaultWindowBag["CanResize"] = False
   ButtonMan.DefaultWindowBag["CanMaximize"] = False
   ButtonMan.DefaultWindowBag["CanClose"] = False
   ButtonMan.DefaultWindowBag["OriginRow"] = SysinfoBag["ScreenHeight"] - 7
   ButtonMan.DefaultWindowBag["OriginCol"] = 1
   ButtonMan.DefaultWindowBag["Title"] = ""
   ButtonMan.DefaultWindowBag["CanMove"] = True
   ButtonMan.DefaultWindowBag["HasShadow"] = True
   ButtonMan.DefaultWindowBag["HasFrame"] = True
   ;--------------------------------------------------------------------------
   ; Default ButtonBar window colors...
   ;--------------------------------------------------------------------------
   Dynarray ButtonMan.DefaultColorBag[]
   ButtonMan.DefaultColorBag[0] = 96 + 0
   ButtonMan.DefaultColorBag[1] = 96 + 0
   ButtonMan.DefaultColorBag[2] = 96 + 0
   ButtonMan.DefaultColorBag[3] = 96 + 0
   ButtonMan.DefaultColorBag[4] = 96 + 0

   ;--------------------------------------------------------------------------
   ; Default ButtonBar methods
   ;--------------------------------------------------------------------------
   Dynarray ButtonMan.MethodBag[]
   ButtonMan.MethodBag["LeftDown"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["RightDown"] = "ButtonMan.UserDialog"
   ButtonMan.MethodBag["BothDown"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["LeftDoubleDown"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["RightDoubleDown"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["BothDoubleDown"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["Up"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag["DoubleUp"] = "ButtonMan.HandleMouse"
   ButtonMan.MethodBag[Asc("WinResize")] = "ButtonMan.Move"

   ButtonMan.HotKeyMethod = "ButtonMan.HandleHotKeys"

   Dynarray ButtonMan.AsciiToHotKey[]
   ButtonMan.AsciiToHotKey["0"] = -129
   ButtonMan.AsciiToHotKey["1"] = -120
   ButtonMan.AsciiToHotKey["2"] = -121
   ButtonMan.AsciiToHotKey["3"] = -122
   ButtonMan.AsciiToHotKey["4"] = -123
   ButtonMan.AsciiToHotKey["5"] = -124
   ButtonMan.AsciiToHotKey["6"] = -125
   ButtonMan.AsciiToHotKey["7"] = -126
   ButtonMan.AsciiToHotKey["8"] = -127
   ButtonMan.AsciiToHotKey["9"] = -128
   ButtonMan.AsciiToHotKey["A"] = -30
   ButtonMan.AsciiToHotKey["B"] = -48
   ButtonMan.AsciiToHotKey["C"] = -46
   ButtonMan.AsciiToHotKey["D"] = -32
   ButtonMan.AsciiToHotKey["E"] = -18
   ButtonMan.AsciiToHotKey["F"] = -33
   ButtonMan.AsciiToHotKey["G"] = -34
   ButtonMan.AsciiToHotKey["H"] = -35
   ButtonMan.AsciiToHotKey["I"] = -23
   ButtonMan.AsciiToHotKey["J"] = -36
   ButtonMan.AsciiToHotKey["K"] = -37
   ButtonMan.AsciiToHotKey["L"] = -38
   ButtonMan.AsciiToHotKey["M"] = -50
   ButtonMan.AsciiToHotKey["N"] = -49
   ButtonMan.AsciiToHotKey["O"] = -24
   ButtonMan.AsciiToHotKey["P"] = -25
   ButtonMan.AsciiToHotKey["Q"] = -16
   ButtonMan.AsciiToHotKey["R"] = -19
   ButtonMan.AsciiToHotKey["S"] = -31
   ButtonMan.AsciiToHotKey["T"] = -20
   ButtonMan.AsciiToHotKey["U"] = -22
   ButtonMan.AsciiToHotKey["V"] = -47
   ButtonMan.AsciiToHotKey["W"] = -17
   ButtonMan.AsciiToHotKey["X"] = -45
   ButtonMan.AsciiToHotKey["Y"] = -21
   ButtonMan.AsciiToHotKey["Z"] = -44

   Dynarray ButtonMan.DefaultPropertyBag[]
   ButtonMan.DefaultPropertyBag["ObjectTag"] = ""
   ButtonMan.DefaultPropertyBag["Orientation"] = "Horizontal" ; or "Vertical"
   ButtonMan.DefaultPropertyBag["Action"] = "Radio" ; or "Toggle" or "Checkbox"
   ButtonMan.DefaultPropertyBag["WrapCount"] = 5
   ButtonMan.DefaultPropertyBag["CanvasColor"] = 96 + 10
   ButtonMan.DefaultPropertyBag["ButtonColor"] = 96 + 10
   ButtonMan.DefaultPropertyBag["SelectedColor"] = 96 + 14
   ButtonMan.DefaultPropertyBag["ShadowColor"] = 96 + 0
   ButtonMan.DefaultPropertyBag["HighlightColor"] = 96 + 15
   ButtonMan.DefaultPropertyBag["HotKeyColor"] = 80 + 12 ; for hotkeys, if any
   ButtonMan.DefaultPropertyBag["SelectedHotKeyColor"] = 80 + 12 ; for hotkeys, if any
   ButtonMan.DefaultPropertyBag["DisableColor"] = 96 + 7 ; for disabled buttons
   ButtonMan.DefaultPropertyBag["ButtonTag"] = ""
   ButtonMan.DefaultPropertyBag["ButtonNumber"] = "" ; Chr(ButtonNumber)
   ButtonMan.DefaultPropertyBag["nButtons"] = BlankNum()
   ButtonMan.DefaultPropertyBag["ButtonBag"] = ""
   ButtonMan.DefaultPropertyBag["HotKeyBag"] = ""
   ButtonMan.DefaultPropertyBag["numRows"] = BlankNum()
   ButtonMan.DefaultPropertyBag["numCols"] = BlankNum()
   ButtonMan.DefaultPropertyBag["HasFrame"] = True
   ButtonMan.DefaultPropertyBag["ButtonStyle"] = "Border" ; or "Shadow" or "Flat"
   ButtonMan.DefaultPropertyBag["MaxStringLength"] = BlankNum()
   ButtonMan.DefaultPropertyBag["UserScript"] = ""
   ButtonMan.DefaultPropertyBag["CanMove"] = True
   ButtonMan.DefaultPropertyBag["MustFloat"] = True

   Dynarray ButtonMan.InterfaceProperties[]
   ButtonMan.InterfaceProperties["Orientation"] = "Horizontal"
   ButtonMan.InterfaceProperties["Action"] = "Radio"
   ButtonMan.InterfaceProperties["ButtonStyle"] = "Border"
   ButtonMan.InterfaceProperties["WrapCount"] = 5
   ButtonMan.InterfaceProperties["CanvasColor"] = 96 + 10
   ButtonMan.InterfaceProperties["ButtonColor"] = 96 + 10
   ButtonMan.InterfaceProperties["HighlightColor"] = 96 + 15
   ButtonMan.InterfaceProperties["ShadowColor"] = 96 + 0
   ButtonMan.InterfaceProperties["SelectedColor"] = 96 + 14
   ButtonMan.InterfaceProperties["HotKeyColor"] = 80 + 12
   ButtonMan.InterfaceProperties["SelectedHotKeyColor"] = 96 + 12
   ButtonMan.InterfaceProperties["DisableColor"] = 96 + 7
   ButtonMan.InterfaceProperties["MustFloat"] = True
   ;--------------------------------------------------------------------------
   ; Button bar instance properties...
   ;--------------------------------------------------------------------------
   Dynarray ButtonMan.Property[] ; ButtonBarTag + Property
   ;--------------------------------------------------------------------------
   ; button properties...
   ;--------------------------------------------------------------------------
   Dynarray ButtonMan.IsButtonTrue[]    ; Index ButtonBarTag + ButtonTag
   Dynarray ButtonMan.ButtonNumber[]    ; Index ButtonBarTag + ButtonTag
   Dynarray ButtonMan.ButtonMethod[]    ; Index ButtonBarTag + ButtonTag
   Dynarray ButtonMan.IsButtonEnabled[]   ; Index ButtonBarTag + ButtonTag
   DynArray ButtonMan.ButtonTag[]       ; Index ButtonBarTag + ButtonNumber
   DynArray ButtonMan.ButtonColOffset[] ; Index ButtonBarTag + ButtonNumber
   DynArray ButtonMan.ButtonRowOffset[] ; Index ButtonBarTag + ButtonNumber
   Dynarray ButtonMan.ButtonLabel[]     ; Index ButtonBarTag + ButtonNumber
   Dynarray ButtonMan.ButtonHotKeyPos[] ; Index ButtonBarTag + ButtonNumber
   Dynarray ButtonMan.HotKeyButtonPtr[] ; Index HotKey
                                             ; Contains Chr(num) + ButtonBarTag
   ButtonMan.MethodToFire = ""
   ButtonMan.ButtonBarTag = ""
   ButtonMan.UserCanConfigure = True
   ButtonMan.ShadowStyleOffset = 0

   ButtonMan.IsActive = True

EndProc ; ButtonMan.Constructor

Proc ButtonMan.Destructor()
   Release Vars
      ; ButtonBar Default Bags...
      ButtonMan.DefaultWindowBag,
      ButtonMan.DefaultColorBag,
      ButtonMan.DefaultPropertyBag,
      ButtonMan.Property,
      ButtonMan.MethodBag,
      ; Button Bags...
      ButtonMan.ButtonTag,
      ButtonMan.ButtonNumber,
      ButtonMan.IsButtonTrue,
      ButtonMan.ButtonColOffset,
      ButtonMan.ButtonRowOffset,
      ButtonMan.ButtonMethod,
      ButtonMan.ButtonLabel,
      ButtonMan.IsButtonEnabled,
      ButtonMan.ButtonHotKeyPos,
      ButtonMan.AsciiToHotKey,
      ButtonMan.HotKeyButtonPtr,
      ButtonMan.MethodToFire,
      ButtonMan.HotKeyMethod,
      ButtonMan.ButtonBarTag,
      ButtonMan.UserCanCongigure,
      ButtonMan.ShadowStyleOffset,
      ButtonMan.IsActive
EndProc ; ButtonMan.Destructor

Proc ButtonMan.MakeButtonBar(PropertyBag,ButtonBag,WindowBag,ColorBag)
   Private
      nButtons,
      WrapCount,
      BarWidth,
      BarHeight,
      ColOffset,
      RowOffset,
      Orientation,
      numRows,
      numCols,
      CurrentWindow,
      WindowHandle,
      IsOnButtonBarWindow,
      WasButtonBarWindow,
      ButtonBarTag,
      WinBag,
      Element,
      ButtonTag,
      OldButtonBag,
      ButtonNumber,
      ButtonPtr,
      ButtonCount,
      MaxStringLength,
      ButtonStyle,
      RowIncrement,
      ColIncrement,
      LabelSpecPtr,
      LabelSpec,
      ButtonLabel,
      ButtonMethod,
      Prefix,
      HotKey,
      Suffix,
      HotKeyBag,
      OldHotKeyBag,
      i,
      UserScript

   If IsAssigned(PropertyBag["UserScript"]) And
      (Not IsAssigned(ButtonMan.ApplyingDialog)) Then
      UserScript = Privdir() + PropertyBag["UserScript"]
      If IsFile(UserScript + ".sc") Then
         Message "Loading custom configuration for button bar"
         Play UserScript
         Message ""
      Endif
   Endif
   ButtonBarTag = PropertyBag["ObjectTag"]
   Window Handle Dialog To CurrentWindow
   If Not IsWindow(CurrentWindow) Then
      CurrentWindow = GetWindow()
   Endif
   IsOnButtonBarWindow = False
   WasButtonBarWindow = False
   ;--------------------------------------------------------------------------
   ; Add default properties not provided with parameters...
   ;--------------------------------------------------------------------------
   ForEach Element In ButtonMan.DefaultPropertyBag
      If Not IsAssigned(PropertyBag[Element]) Then
         PropertyBag[Element] = ButtonMan.DefaultPropertyBag[Element]
      Endif
   EndForEach
   ;--------------------------------------------------------------------------
   ; Add default window attributes not provided with parameters...
   ;--------------------------------------------------------------------------
   ForEach Element In ButtonMan.DefaultWindowBag
      If Not IsAssigned(WindowBag[Element]) Then
         WindowBag[Element] = ButtonMan.DefaultWindowBag[Element]
      Endif
   EndForEach
   If PropertyBag["MustFloat"] Then
      WindowBag["Floating"] = True
   Endif
   PropertyBag["CanMove"] = WindowBag["CanMove"]
   PropertyBag["HasFrame"] = WindowBag["HasFrame"]
   ;--------------------------------------------------------------------------
   ; Add default window colors not provided with parameters...
   ;--------------------------------------------------------------------------
   ForEach Element In ButtonMan.DefaultColorBag
      If Not IsAssigned(ColorBag[Element]) Then
         ColorBag[Element] = PropertyBag["ShadowColor"]
      Endif
   EndForEach
   ;--------------------------------------------------------------------------
   ; If the buttonbar already exists in some form, then capture the
   ; co-ordinates and destroy it...
   ;--------------------------------------------------------------------------
   If IsAssigned(EventMan.ObjectTagWindow[ButtonBarTag]) Then
      WasButtonBarWindow = True
      IsOnButtonBarWindow =
         (EventMan.ObjectTagWindow[ButtonBarTag] = GetWindow())
      Window Select EventMan.ObjectTagWindow[ButtonBarTag]
      Window GetAttributes
         EventMan.ObjectTagWindow[ButtonBarTag] To WinBag
      WindowBag["OriginRow"] = WinBag["OriginRow"]
      WindowBag["OriginCol"] = WinBag["OriginCol"]
      Window Close
      Release Vars
         EventMan.WindowObjectTag[EventMan.ObjectTagWindow[ButtonBarTag]]
      Dynarray OldButtonBag[]
      PackMan.UnpackAnyArrayFrom(
         ButtonMan.Property[ButtonBarTag + "ButtonBag"],
         OldButtonBag)
      ForEach ButtonTag in OldButtonBag
         ButtonNumber = Substr(OldButtonBag[ButtonTag],1,1)
         Release Vars
            ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag],
            ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag],
            ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag],
            ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber],
            ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag],
            ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]
      EndForeach
      Dynarray OldHotKeyBag[]
      PackMan.UnpackAnyArrayFrom(
         ButtonMan.Property[ButtonBarTag + "HotKeyBag"],
         OldHotKeyBag)
      EventMan.ClearHandlersWith(OldHotKeyBag)
      ForEach Element in OldHotKeyBag
         Release Vars ButtonMan.HotKeyButtonPtr[Element]
      EndForEach
   Endif
   ;--------------------------------------------------------------------------
   ; Update values in the property bag...
   ;--------------------------------------------------------------------------
   PropertyBag["nButtons"] = Dynarraysize(ButtonBag)
   PropertyBag["ButtonBag"] = PackMan.PackAnyArrayFrom(ButtonBag)
   ;--------------------------------------------------------------------------
   ; Use them to set up new button bar information...
   ;--------------------------------------------------------------------------
   If Not WasButtonBarWindow Then ; new object...
      Foreach Element in PropertyBag
         ButtonMan.Property[ButtonBarTag + Element] =
            PropertyBag[Element]
      EndForEach
      ButtonMan.MethodBag["ObjectTag"] = ButtonBarTag
      EventMan.SetHandlersFrom(ButtonMan.MethodBag)
      Release Vars ButtonMan.MethodBag["ObjectTag"]
   Else ; updated object...
      ForEach Element in PropertyBag
         ButtonMan.Property[ButtonBarTag + Element] =  PropertyBag[Element]
      EndForEach
   Endif
   ;--------------------------------------------------------------------------
   ; Move back to the starting window if it still exists...
   ;--------------------------------------------------------------------------
   If IsWindow(CurrentWindow) Then
      Window Select CurrentWindow
   Endif
   ;--------------------------------------------------------------------------
   ; Set up most data for the new button bar buttons, and capture maximum
   ; buttontag length (MaxStringLength) at the same time...
   ;--------------------------------------------------------------------------
   MaxStringLength = 0
   Dynarray HotKeyBag[]
   ForEach ButtonTag in ButtonBag
      ButtonNumber = Substr(ButtonBag[ButtonTag],1,1)
      ButtonMethod = Substr(ButtonBag[ButtonTag],2,255)
      If Match(ButtonMethod,"..,..",ButtonMethod,LabelSpec) Then
         If Match(LabelSpec,"..~@~..",Prefix,HotKey,Suffix) Then
            ButtonLabel = Prefix + HotKey + Suffix
            ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] =
               Search("~",LabelSpec) + 1
            HotKeyBag[ButtonMan.AsciiToHotKey[Upper(HotKey)]] =
               ButtonMan.HotKeyMethod
            ButtonMan.HotKeyButtonPtr[ButtonMan.AsciiToHotKey[Upper(HotKey)]] =
               ButtonNumber + ButtonBarTag
         Else
            ButtonLabel = LabelSpec
         Endif
      Else
         ButtonLabel = ButtonTag
      Endif
      ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag] = ButtonMethod
      ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber] = ButtonLabel
      ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag] = True
      ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber] = ButtonTag
      ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag] = ButtonNumber
      ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] = False
      MaxStringLength = Max(MaxStringLength,Len(ButtonLabel))
   EndForEach
   ButtonMan.Property[ButtonBarTag + "MaxStringLength"] = MaxStringLength
   ButtonMan.Property[ButtonBarTag + "HotKeyBag"] =
      PackMan.PackAnyArrayFrom(HotKeyBag)
   EventMan.SetHandlersFrom(HotKeyBag)
   ;--------------------------------------------------------------------------
   ; Initialize stuff in preparation for calculating positions of the
   ; buttons...
   ;--------------------------------------------------------------------------
   Orientation = PropertyBag["Orientation"]
   nButtons = PropertyBag["nButtons"]
   WrapCount = PropertyBag["WrapCount"]
   ButtonMan!SetNumRowsAndCols()
   ButtonStyle = PropertyBag["ButtonStyle"]
   Switch
      Case ButtonStyle = "Border":
         RowIncrement = 3
         ColIncrement = 4
      Case ButtonStyle = "Shadow":
         RowIncrement = 2
         ColIncrement = 4
      Case ButtonStyle = "Flat":
         RowIncrement = 1
         ColIncrement = 2
   EndSwitch
   BarHeight = numRows * RowIncrement
   If WindowBag["HasFrame"] Then
      BarWidth = Max(14,numCols * (MaxStringLength + ColIncrement))
   Else
      BarWidth = numCols * (MaxStringLength + ColIncrement)
   Endif
   ButtonMan!CalcOffsets()
   ButtonMan!DimensionWindow()
   ButtonMan!CreateWindow()
   If IsOnButtonBarWindow Then
      Window Select WindowHandle
   Else
      If IsWindow(CurrentWindow) Then
         Window Select CurrentWindow
      Endif
   Endif

EndProc ; ButtonMan.MakeButtonBar

;--------------------------------------------------------------------------
; Requires Orientation, WrapCount, nButtons
;--------------------------------------------------------------------------
Proc ButtonMan!SetNumRowsAndCols()
   If Orientation = "Horizontal" Then
      numRows = Int(nButtons/WrapCount) +
         IIf(Mod(nButtons,Wrapcount) = 0,0,1)
      numCols = Min(WrapCount,nButtons)
   Else
      numCols = Int(nButtons/WrapCount) +
         IIf(Mod(nButtons,Wrapcount) = 0,0,1)
      numRows = Min(WrapCount,nButtons)
   Endif
   ButtonMan.Property[ButtonBarTag + "numRows"] = numRows
   ButtonMan.Property[ButtonBarTag + "numCols"] = numCols
EndProc ; ButtonMan!SetNumRowsAndCols

;--------------------------------------------------------------------------
; Requires Orientation, nButtons, ButtonBarTag, RowIncrement, ColIncrement,
; MaxStringLength
;--------------------------------------------------------------------------
Proc ButtonMan!CalcOffsets()
   Private
      ColOffset,
      RowOffset,
      MinColOffset,
      MinRowOffset
   ;--------------------------------------------------------------------------
   ; Calculate  Row and Column offsets for each button...
   ;--------------------------------------------------------------------------
   If Not ButtonMan.Property[ButtonBarTag + "HasFrame"] Then
      If ButtonMan.Property[ButtonBarTag + "Orientation"] = "Horizontal" Then
         If WindowBag["CanMove"] Then
            MinColOffset = 1
         Else
            MinColOffset = 0
         Endif
         If ButtonMan.Property[ButtonBarTag + "ButtonStyle"] = "Shadow" Then
            MinRowOffset = ButtonMan.ShadowStyleOffset
         Else
            MinRowOffset = 0
         Endif
      Else
         MinColOffset = 0
         If WindowBag["CanMove"] Then
            MinRowOffset = 1
         Else
            MinRowOffset = 0
         Endif
      Endif
   Else
      MinColOffset = 0
      MinRowOffset = 0
   Endif
   ColOffset = MinColOffset
   RowOffset = MinRowOffset
   ButtonCount = 0
   If Orientation = "Horizontal" Then
      For i From 1 to nButtons
         ButtonCount = ButtonCount + 1
         ButtonNumber = Chr(i)
         ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber] = ColOffset
         ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber] = RowOffset
         If ButtonCount = WrapCount Then
            ColOffset = MinColOffset
            RowOffset = RowOffset + RowIncrement
            Buttoncount = 0
         Else
            ColOffset = ColOffset + MaxStringLength + ColIncrement
         Endif
      EndFor
   Else
      For i From 1 to nButtons
         ButtonCount = ButtonCount + 1
         ButtonNumber = Chr(i)
         ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber] = ColOffset
         ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber] = RowOffset
         If ButtonCount = WrapCount Then
            RowOffset = MinRowOffset
            ColOffset = ColOffset + MaxStringLength + ColIncrement
            Buttoncount = 0
         Else
            RowOffset = RowOffset + RowIncrement
         Endif
      EndFor
   Endif
EndProc ; ButtonMan!CalcOffsets

;--------------------------------------------------------------------------
; Requires BarHeight, BarWidth, and ButtonBarTag
;--------------------------------------------------------------------------
Proc ButtonMan!DimensionWindow()
   Private
      MoveIncrement
   ;--------------------------------------------------------------------------
   ; Set WindowBag Height and Width for Canvas and Window
   ;--------------------------------------------------------------------------
   If ButtonMan.Property[ButtonBarTag + "HasFrame"] Then
      WindowBag["Height"] = BarHeight + 2
      WindowBag["Width"] = BarWidth + 2
      WindowBag["CanvasHeight"] = BarHeight
      WindowBag["CanvasWidth"] = BarWidth
   Else
      If WindowBag["CanMove"] Then
         MoveIncrement = 1
      Else
         MoveIncrement = 0
      Endif
      If ButtonMan.Property[ButtonBarTag + "Orientation"] = "Horizontal" Then
         WindowBag["Width"] = BarWidth + MoveIncrement
         WindowBag["CanvasWidth"] = BarWidth + MoveIncrement
         If ButtonMan.Property[ButtonBarTag + "ButtonStyle"] = "Shadow" Then
            WindowBag["Height"] = BarHeight + ButtonMan.ShadowStyleOffset
            WindowBag["CanvasHeight"] = BarHeight + ButtonMan.ShadowStyleOffset
         Else
            WindowBag["Height"] = BarHeight
            WindowBag["CanvasHeight"] = BarHeight
         Endif
      Else
         WindowBag["Width"] = BarWidth
         WindowBag["Height"] = BarHeight + MoveIncrement
         WindowBag["CanvasWidth"] = BarWidth
         WindowBag["CanvasHeight"] = BarHeight + MoveIncrement
      Endif
   Endif
EndProc ; ButtonMan!DimensionWindow

;--------------------------------------------------------------------------
; Requires WindowBag, ColorBag, ButtonBarTag: Sets WindowHandle
;--------------------------------------------------------------------------
Proc ButtonMan!CreateWindow()
   Private
      OriginRow,
      OriginCol
   ;--------------------------------------------------------------------------
   ; Create the button bar window...
   ;--------------------------------------------------------------------------
   If WindowBag["OriginRow"] < 1 Then
      WindowBag["OriginRow"] = 1
   Endif
   OriginRow = WindowBag["OriginRow"]
   OriginCol = WindowBag["OriginCol"]
   WindowBag["Style"] = ButtonMan.Property[ButtonBarTag + "CanvasColor"]
   Window Create Attributes WindowBag To WindowHandle
   Window SetColors WindowHandle From ColorBag
   If IsAssigned(EventMan.ObjectTagWindow[ButtonBarTag]) Then
      Release vars
         EventMan.WindowObjectTag[EventMan.ObjectTagWindow[ButtonBarTag]]
   Endif
   EventMan.ObjectTagWindow[ButtonBarTag] = WindowHandle
   EventMan.WindowObjectTag[WindowHandle] = ButtonBarTag
   ButtonMan!MakeButtons()
EndProc ; ButtonMan!CreateWindow

Proc ButtonMan!MakeButtons()
   Private
      MaxStringLength,
      ButtonBag,
      ButtonLabel
   ;--------------------------------------------------------------------------
   ; Paint the buttons on the window...
   ;--------------------------------------------------------------------------
   If ButtonStyle = "Flat" Then
      MaxStringLength = ButtonMan.Property[ButtonBarTag + "MaxStringLength"]
   Else
      MaxStringLength = ButtonMan.Property[ButtonBarTag + "MaxStringLength"] + 2
   Endif
   SetCanvas WindowHandle
   If (Not ButtonMan.Property[ButtonBarTag + "HasFrame"]) And
      ButtonMan.Property[ButtonBarTag + "CanMove"] Then
      Canvas off
      If ButtonMan.Property[ButtonBarTag + "Orientation"] = "Horizontal" Then
         PaintCanvas
            Fill "\007"
            Attribute ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            0,0,WindowBag["Height"] - 1,0
      Else
         PaintCanvas
            Fill " \007"
            Attribute ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            0,0,0,WindowBag["Width"] - 2
      Endif
      Canvas On
   Endif
   ;---------------------------------------------------------------------------
   ; Save Checkbox buttons to reset them after drawing them...
   ;---------------------------------------------------------------------------
   If ButtonMan.Property[ButtonBarTag + "Action"] = "Checkbox" Then
      Dynarray ButtonBag[]
      For ButtonPtr From 1 To nButtons
         ButtonNumber = Chr(ButtonPtr)
         ButtonTag = ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber]
         ButtonBag[ButtonTag] = ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag]
      EndFor
   Endif
   Switch
      Case ButtonStyle = "Border":
         Canvas off
         For ButtonPtr From 1 to nButtons
            ButtonNumber = Chr(ButtonPtr)
            ButtonLabel = ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber]
            Frame Single From
               ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber],
               ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber] To
               ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber] + 2,
               ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber] +
                  MaxStringLength + 1
            @ButtonMan.ButtonRowoffset[ButtonBarTag + ButtonNumber] + 1,
               ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber] + 2
            ?? ButtonLabel
            ButtonTag = ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber]
            ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
         EndFor
         Canvas on
      Case ButtonStyle = "Shadow":
         Canvas off
         For ButtonPtr From 1 to nButtons
            ButtonNumber = Chr(ButtonPtr)
            ButtonTag = ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber]
            ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
         EndFor
         Canvas on
      Case ButtonStyle = "Flat":
         Canvas off
         For ButtonPtr From 1 to nButtons
            ButtonNumber = Chr(ButtonPtr)
            ButtonLabel = ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber]
            @ButtonMan.ButtonRowoffset[ButtonBarTag + ButtonNumber],
               ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
            ?? " " + ButtonLabel +
               Fill(" ",MaxStringLength - Len(ButtonLabel)) + " "
            ButtonTag = ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber]
            ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
         EndFor
         Canvas on
   EndSwitch
   Switch
      Case ButtonMan.Property[ButtonBarTag + "Action"] = "Radio":
         If IsBlank(ButtonMan.Property[ButtonBarTag + "ButtonNumber"]) Then
            ButtonMan.Property[ButtonBarTag + "ButtonNumber"] = Chr(1)
            ButtonMan.Property[ButtonBarTag + "ButtonTag"] =
               ButtonMan.ButtonTag[ButtonBarTag + Chr(1)]
         Endif
         ButtonTag = ButtonMan.Property[ButtonBarTag + "ButtonTag"]
         ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
      Case ButtonMan.Property[ButtonBarTag + "Action"] = "Checkbox":
         ForEach ButtonTag in ButtonBag
            If ButtonBag[ButtonTag] Then
               ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
            Endif
         EndForEach
   EndSwitch
EndProc ; ButtonMan!MakeButtons

Proc ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
   Private
      ButtonNumber,
      MaxStringLength,
      RowOffset,
      ColOffset,
      ButtonStyle,
      ButtonLabel

   ButtonNumber = ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag]
   ButtonLabel = ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber]
   ButtonStyle = ButtonMan.Property[ButtonBarTag + "ButtonStyle"]
   MaxStringLength = ButtonMan.Property[ButtonBarTag + "MaxStringLength"] + 2
   RowOffset = ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber]
   ColOffset = ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
   SetCanvas EventMan.ObjectTagWindow[ButtonBarTag]
   Switch
      Case ButtonStyle = "Border":
         PaintCanvas Border Attribute
            ButtonMan.Property[ButtonBarTag + "HighLightColor"]
            RowOffset, ColOffset, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset + 2, ColOffset + 1, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset, ColOffset + MaxStringLength + 1, RowOffset + 2,
               ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ButtonColor"]
            RowOffset + 1, ColOffset + 1, RowOffset + 1, ColOffset + MaxStringLength
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "HotKeyColor"]
               RowOffset + 1,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber],
               RowOffset + 1,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]
         Endif
      Case ButtonStyle = "Shadow":
         @ButtonMan.ButtonRowoffset[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
         ?? "  " + ButtonLabel +
            Fill(" ",MaxStringLength - Len(ButtonLabel) - 2) + " "
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ButtonColor"]
            RowOffset, ColOffset + 1,
            RowOffset, ColOffset + MaxStringLength
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "HotKeyColor"]
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber],
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]
         Endif
         PaintCanvas Fill "" Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset + 1, ColOffset + 2,
            RowOffset + 1, ColOffset + MaxStringLength + 1
         PaintCanvas Fill "" Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset, ColOffset + MaxStringLength + 1,
            RowOffset, ColOffset + MaxStringLength + 1
      Case ButtonStyle = "Flat":
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ButtonColor"]
            RowOffset, ColOffset, RowOffset, ColOffset + MaxStringLength - 1
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "HotKeyColor"]
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] - 1,
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] - 1
         Endif
   EndSwitch
   ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] = False
EndProc ; ButtonMan.PaintOutButton

Proc ButtonMan.PaintDisableButton(ButtonBarTag,ButtonTag)
   Private
      ButtonNumber,
      MaxStringLength,
      RowOffset,
      ColOffset,
      ButtonStyle,
      ButtonLabel

   ButtonNumber = ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag]
   ButtonLabel = ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber]
   ButtonStyle = ButtonMan.Property[ButtonBarTag + "ButtonStyle"]
   MaxStringLength = ButtonMan.Property[ButtonBarTag + "MaxStringLength"] + 2
   RowOffset = ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber]
   ColOffset = ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
   SetCanvas EventMan.ObjectTagWindow[ButtonBarTag]
   Switch
      Case ButtonStyle = "Border":
         PaintCanvas Border Attribute
            ButtonMan.Property[ButtonBarTag + "HighLightColor"]
            RowOffset, ColOffset, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset + 2, ColOffset + 1, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset, ColOffset + MaxStringLength + 1, RowOffset + 2,
               ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "DisableColor"]
            RowOffset + 1, ColOffset + 1, RowOffset + 1, ColOffset + MaxStringLength
      Case ButtonStyle = "Shadow":
         @ButtonMan.ButtonRowoffset[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
         ?? "  " + ButtonLabel +
            Fill(" ",MaxStringLength - Len(ButtonLabel) - 2) + " "
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "DisableColor"]
            RowOffset, ColOffset + 1,
            RowOffset, ColOffset + MaxStringLength
         PaintCanvas Fill "" Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset + 1, ColOffset + 2,
            RowOffset + 1, ColOffset + MaxStringLength + 1
         PaintCanvas Fill "" Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset, ColOffset + MaxStringLength + 1,
            RowOffset, ColOffset + MaxStringLength + 1
      Case ButtonStyle = "Flat":
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "DisableColor"]
            RowOffset, ColOffset, RowOffset, ColOffset + MaxStringLength - 1
   EndSwitch
   ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] = False
EndProc ; ButtonMan.PaintDisableButton

Proc ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
   Private
      ButtonNumber,
      MaxStringLength,
      RowOffset,
      ColOffset,
      ButtonStyle,
      ButtonLabel

   ButtonNumber = ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag]
   ButtonLabel = ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber]
   ButtonStyle = ButtonMan.Property[ButtonBarTag + "ButtonStyle"]
   MaxStringLength = ButtonMan.Property[ButtonBarTag + "MaxStringLength"] + 2
   RowOffset = ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber]
   ColOffset = ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
   SetCanvas EventMan.ObjectTagWindow[ButtonBarTag]
   Switch
      Case ButtonStyle = "Border":
         PaintCanvas Border Attribute
            ButtonMan.Property[ButtonBarTag + "ShadowColor"]
            RowOffset, ColOffset, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag+ "HighlightColor"]
            RowOffset + 2, ColOffset + 1, RowOffset + 2, ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "HighlightColor"]
            RowOffset, ColOffset + MaxStringLength + 1, RowOffset + 2,
               ColOffset + MaxStringLength + 1
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "SelectedColor"]
            RowOffset + 1, ColOffset + 1, RowOffset + 1, ColOffset + MaxStringLength
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "SelectedHotKeyColor"]
               RowOffset + 1,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber],
               RowOffset + 1,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]
         Endif
      Case ButtonStyle = "Shadow":
         @ButtonMan.ButtonRowoffset[ButtonBarTag + ButtonNumber],
            ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber]
         ?? "   " + ButtonLabel +
            Fill(" ",MaxStringLength - Len(ButtonLabel) - 2) + " "
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "SelectedColor"]
            RowOffset, ColOffset + 2,
            RowOffset, ColOffset + MaxStringLength + 1
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "SelectedHotKeyColor"]
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] + 1,
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] + 1
         Endif
         PaintCanvas Fill " " Attribute
            ButtonMan.Property[ButtonBarTag + "CanvasColor"]
            RowOffset + 1, ColOffset + 2,
            RowOffset + 1, ColOffset + MaxStringLength + 1
         PaintCanvas Fill " " Attribute
            ButtonMan.Property[ButtonBarTag + "CanvasColor"]
            RowOffset, ColOffset + 1,
            RowOffset + 1, ColOffset + 1
      Case ButtonStyle = "Flat":
         PaintCanvas Attribute
            ButtonMan.Property[ButtonBarTag + "SelectedColor"]
            RowOffset, ColOffset, RowOffset, ColOffset + MaxStringLength - 1
         If IsAssigned(ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]) Then
            PaintCanvas Attribute
               ButtonMan.Property[ButtonBarTag + "SelectedHotKeyColor"]
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] - 1,
               RowOffset,ColOffset +
                  ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber] - 1
         Endif
   EndSwitch
   ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] = True
EndProc ; ButtonMan.PaintInButton

Proc ButtonMan.HandleMouse()
   Private
      MethodName,
      ReturnCode
   If EventMan.EventBag["Action"] = "DOWN" Then
      ReturnCode = ButtonMan.SetButton(EventMan.ObjectTag)
      If ReturnCode = 0 Then
         ExecEvent EventMan.EventBag
         Return 1
      Else
         If ReturnCode = 1 Then
            ReturnCode = EventMan.CompleteMouseSequence()
            Return ReturnCode
         Else
            EventMan.ReturnFromMouseMode()
            Return ReturnCode
         Endif
      Endif
   Else
      ButtonMan.ResetButton(EventMan.ObjectTag)
      If Not IsBlank(ButtonMan.MethodToFire) Then
         MethodName = ButtonMan.MethodToFire
         ButtonMan.MethodToFire = ""
         ExecProc MethodName
         Return RetVal
      Else
         Return 1
      Endif
   Endif
EndProc ; ButtonMan.HandleMouse

Proc ButtonMan.HandleHotKeys()
   Private
      MethodName,
      ReturnCode,
      ButtonNumber,
      ButtonBarTag,
      ButtonTag,
      ButtonPtr

   If Not IsAssigned(ButtonMan.HotKeyButtonPtr[EventMan.EventTag]) Then
      Message "Invalid hotkey"
      Return 1
   Endif
   ButtonPtr = ButtonMan.HotKeyButtonPtr[EventMan.EventTag]
   ButtonNumber = Substr(ButtonPtr,1,1)
   ButtonBarTag = Substr(ButtonPtr,2,255)
   ButtonTag = ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber]
   ButtonMan.SelectButton(ButtonBarTag,ButtonTag)
   ButtonMan.ResetButton(ButtonBarTag)
   If Not IsBlank(ButtonMan.MethodToFire) Then
      MethodName = ButtonMan.MethodToFire
      ButtonMan.MethodToFire = ""
      ExecProc MethodName
      Return RetVal
   Else
      Return 1
   Endif
EndProc ; ButtonMan.HandleHotKeys

Proc ButtonMan.Move()
   Private
      WindowBag
   WinResize
   Window GetAttributes EventMan.TargetWindow To WindowBag
   If WindowBag["OriginRow"] < 1 Then
      WindowBag["OriginRow"] = 1
      Window SetAttributes EventMan.TargetWindow From WindowBag
   Endif
   Return 1
EndProc ; ButtonMan.Move

Proc ButtonMan.SetButton(ButtonBarTag)
   Private
      ScreenRow,
      ScreenCol,
      Found,
      ButtonPtr,
      ButtonNumber,
      ButtonTag,
      MouseRow,
      MouseCol,
      WindowBag,
      CurrentWindow,
      ButtonBarWindow,
      OriginRow,
      OriginCol,
      EventBag,
      SearchRowOffset,
      SearchColOffset,
      ButtonStyle,
      RowIncrement,
      ColIncrement

   ButtonBarWindow = EventMan.ObjectTagWindow[ButtonBarTag]
   If ButtonMan.Property[ButtonBarTag + "Action"] = "Toggle" Then
      ButtonMan.Property[ButtonBarTag + "ButtonNumber"] = ""
      ButtonMan.Property[ButtonBarTag + "ButtonTag"] = ""
   Endif
   ScreenRow = EventMan.EventBag["Row"]
   ScreenCol = EventMan.EventBag["Col"]
   LocalizeEvent EventMan.EventBag
   MouseRow = EventMan.EventBag["Row"]
   MouseCol = EventMan.EventBag["Col"]
   EventMan.EventBag["Row"] = ScreenRow
   EventMan.EventBag["Col"] = ScreenCol
   Window GetAttributes ButtonBarWindow To WindowBag
   If (WindowBag["HasFrame"] And MouseRow <> 0 And
      MouseRow <> WindowBag["Height"] - 1 And
      MouseCol <> 0 And
      MouseCol <> WindowBag["Width"] - 1)
      Or (
      (Not WindowBag["HasFrame"]) And
      (((ButtonMan.Property[ButtonBarTag + "Orientation"] = "Horizontal") And
      MouseCol <> 0) or
      ((ButtonMan.Property[ButtonBarTag + "Orientation"] = "Vertical") And
      MouseRow <> 0))
      )
      Then
      If WindowBag["HasFrame"] Then
         SearchRowOffset = 0 + WindowBag["ScrollRow"]
         SearchColOffset = 0 + WindowBag["ScrollCol"]
      Else
         SearchRowOffset = 1 + WindowBag["ScrollRow"]
         SearchColOffset = 1 + WindowBag["ScrollCol"]
      Endif
      Found = False
      For ButtonPtr From ButtonMan.Property[ButtonBarTag + "nButtons"]
         To 1 Step -1
         ButtonNumber = Chr(ButtonPtr)
         If MouseRow + SearchRowOffset > ButtonMan.ButtonRowOffset[
               ButtonBarTag + ButtonNumber] And
            MouseCol + SearchColOffset > ButtonMan.ButtonColOffset[
               ButtonBarTag + ButtonNumber] Then
            Found = True
            Quitloop
         Endif
      EndFor
      If Found Then
         ButtonStyle = ButtonMan.Property[ButtonBarTag + "ButtonStyle"]
         Switch
            Case ButtonStyle = "Border":
               RowIncrement = 3
               ColIncrement = 4
            Case ButtonStyle = "Shadow":
               RowIncrement = 2
               ColIncrement = 4
            Case ButtonStyle = "Flat":
               RowIncrement = 1
               ColIncrement = 2
         EndSwitch
         Found =
            (MouseRow + SearchRowOffset <= ButtonMan.ButtonRowOffset[
               ButtonBarTag + ButtonNumber] + RowIncrement And
             MouseCol + SearchColOffset <= ButtonMan.ButtonColOffset[
               ButtonBarTag + ButtonNumber] +
               ButtonMan.Property[ButtonBarTag + "MaxStringLength"] +
                  ColIncrement)
      Endif

      If Found Then
         ButtonMan.SelectButton(ButtonBarTag,
            ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber])
      Endif
      Return 1
   Else
      If WindowBag["HasFrame"] Then
         If IsAssigned(Appman.IsActive) Then
            If GetWindow() <> EventMan.ObjectTagWindow[ButtonBarTag] Then
               AppMan.TargetTag = ButtonBarTag
               Return AppMan$QuitEventLoop
            Else
               ButtonMan!UserMoveWindow()
               Return 1
            Endif
         Else
            ButtonMan!UserMoveWindow()
            Return 1
         Endif
      Else
         If ButtonMan.Property[ButtonBarTag + "CanMove"] Then
            OriginRow = WindowBag["OriginRow"]
            OriginCol = WindowBag["OriginCol"]
            MouseRow = EventMan.EventBag["Row"]
            MouseCol = EventMan.EventBag["Col"]
            While True
               GetEvent To EventBag
               If EventBag["Type"] = "MOUSE" Then
                  If EventBag["Action"] = "MOVE" Then
                     Window Move ButtonBarWindow To
                        OriginRow + (EventBag["Row"] - MouseRow),
                        OriginCol + (EventBag["Col"] - MouseCol)
                  Endif
                  If EventBag["Action"] = "UP" Then
                     Quitloop
                  Endif
               Endif
            EndWhile
            Window GetAttributes EventMan.TargetWindow To WindowBag
            If WindowBag["OriginRow"] < 1 Then
               WindowBag["OriginRow"] = 1
               Window SetAttributes EventMan.TargetWindow From WindowBag
            Endif
         Else
            EventMan.ReturnFromMouseMode()
         Endif
         Return 1
      Endif
   Endif
EndProc ; ButtonMan.SetButton

Proc ButtonMan!UserMoveWindow()
   Private
      WindowBag
   ExecEvent EventMan.EventBag
   Window GetAttributes EventMan.TargetWindow To WindowBag
   If WindowBag["OriginRow"] < 1 Then
      WindowBag["OriginRow"] = 1
      Window SetAttributes EventMan.TargetWindow From WindowBag
   Endif
EndProc ; ButtonMan!UserMoveWindow

Proc ButtonMan.EnableButton(ButtonBarTag,ButtonTag,Boolean)
   Private
      WindowBag
   If ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag] = Boolean Then
      Return True
   Endif
   If (Not Boolean) And ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] Then
      Return False
   Endif
   Canvas Off
   Window GetAttributes EventMan.ObjectTagWindow[ButtonBarTag] To WindowBag
   ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag] = Boolean
   If Boolean Then
      ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
   Else
      ButtonMan.PaintDisableButton(ButtonBarTag,ButtonTag)
   Endif
   Window Scroll EventMan.ObjectTagWindow[ButtonBarTag] To
      WindowBag["ScrollRow"],WindowBag["ScrollCol"]
   Canvas On
   Return True
EndProc ; ButtonMan.EnableButton

Proc ButtonMan.SelectButton(ButtonBarTag,ButtonTag)
   Private
      ButtonPtr,
      RowOffset,
      ColOffset,
      ButtonNumber,
      NewButtonNumber,
      WindowBag

   ButtonMan.ButtonBarTag = ButtonBarTag
   If Not ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag] Then
      Message "Button disabled"
      Return False
   Endif
   Window GetAttributes EventMan.ObjectTagWindow[ButtonBarTag] To WindowBag
   SetCanvas EventMan.ObjectTagWindow[ButtonBarTag]
   Canvas Off
   Switch
      Case ButtonMan.Property[ButtonBarTag + "Action"] = "Radio":
      ;----------------------------------------------------------
         ; Push the selected button in...
         ;-------------------------------------------------------
         If Not ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] Then
            ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
            ButtonMan.MethodToFire =
               ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag]
         Endif
         ;----------------------------------------------------------
         ; If the previous button was different, then let it out...
         ;----------------------------------------------------------
         If Upper(ButtonTag) <>
            ButtonMan.Property[ButtonBarTag + "ButtonTag"] Then
            NewButtonTag = ButtonTag
            If ButtonMan.Property[ButtonBarTag + "ButtonTag"] <> "" Then
               ButtonTag = ButtonMan.Property[ButtonBarTag + "ButtonTag"]
            Else
               ButtonTag = ""
            Endif
            ButtonMan.Property[ButtonBarTag + "ButtonTag"] = NewButtonTag
            ButtonMan.Property[ButtonBarTag + "ButtonNumber"] =
               ButtonMan.ButtonNumber[ButtonBarTag + NewButtonTag]
            If ButtonTag <> "" Then
               ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
            Endif
         Endif
      Case ButtonMan.Property[ButtonBarTag + "Action"] = "Toggle":
      ;-----------------------------------------------------------
         ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
         ButtonMan.Property[ButtonBarTag + "ButtonTag"] = ButtonTag
         ButtonMan.Property[ButtonBarTag + "ButtonNumber"] =
            ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag]
         ButtonMan.MethodToFire =
            ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag]
      Case ButtonMan.Property[ButtonBarTag + "Action"] = "Checkbox":
      ;-------------------------------------------------------------
         If Not ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag] Then
            ButtonMan.PaintInButton(ButtonBarTag,ButtonTag)
            ButtonMan.Property[ButtonBarTag + "ButtonTag"] = ButtonTag
            ButtonMan.Property[ButtonBarTag + "ButtonNumber"] =
               ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag]
            ButtonMan.MethodToFire =
               ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag]
         Else
            ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
         Endif
   EndSwitch
   Window Scroll EventMan.ObjectTagWindow[ButtonBarTag] To
      WindowBag["ScrollRow"],WindowBag["ScrollCol"]
   Canvas On
   Return True
EndProc ; ButtonMan.SelectButton

Proc ButtonMan.ResetButton(ButtonBarTag)
   Private
      ButtonBarTag,
      ButtonTag,
      ButtonNumber,
      RowOffset,
      ColOffset,
      WindowBag

   If ButtonMan.Property[ButtonBarTag + "Action"] = "Toggle" Then
      ButtonTag = ButtonMan.Property[ButtonBarTag + "ButtonTag"]
      If ButtonTag <> "" Then
         Window GetAttributes EventMan.ObjectTagWindow[ButtonBarTag] To
            WindowBag
         ButtonMan.PaintOutButton(ButtonBarTag,ButtonTag)
         Window Scroll EventMan.ObjectTagWindow[ButtonBarTag] To
            WindowBag["ScrollRow"],WindowBag["ScrollCol"]
      Endif
   Endif
EndProc ; ButtonMan.ResetButton

Proc ButtonMan.DestroyButtonBar(ButtonBarTag)
   Private
      CurrentWindow,
      ButtonWindow,
      ButtonBag,
      ButtonTag,
      HotKeyBag,
      ButtonNumber,
      Element,
      EventBag,
      WindowBag
   CurrentWindow = GetWindow()
   If IsAssigned(EventMan.ObjectTagWindow[ButtonBarTag]) And
      IsWindow(EventMan.ObjectTagWindow[ButtonBarTag]) Then
      ButtonWindow = EventMan.ObjectTagWindow[ButtonBarTag]
      Window Select ButtonWindow
      ; If current "window" was the desktop, then GetWindow() still returns
      ; 0 at this point.
      If RetVal Then
         If Not IsWindow(GetWindow()) Then ; must be in menu system
            Menu Esc
         Endif
      Else
         Debug ; Unexepected Window Select failure
      Endif
      Window Close
      Release Vars
         EventMan.WindowObjectTag[EventMan.ObjectTagWindow[ButtonBarTag]]
      Release Vars
         EventMan.ObjectTagWindow[ButtonBarTag]
   Else
      Return
   Endif
   If IsWindow(CurrentWindow) Then
      Window Select CurrentWindow
   Endif
   Dynarray ButtonBag[]
   PackMan.UnpackAnyArrayFrom(
      ButtonMan.Property[ButtonBarTag + "ButtonBag"],
      ButtonBag)
   ForEach ButtonTag in ButtonBag
      ButtonNumber = Substr(ButtonBag[ButtonTag],1,1)
      Release Vars
         ButtonMan.ButtonTag[ButtonBarTag + ButtonNumber],
         ButtonMan.ButtonNumber[ButtonBarTag + ButtonTag],
         ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag],
         ButtonMan.ButtonRowOffset[ButtonBarTag + ButtonNumber],
         ButtonMan.ButtonColOffset[ButtonBarTag + ButtonNumber],
         ButtonMan.ButtonMethod[ButtonBarTag + ButtonTag],
         ButtonMan.ButtonLabel[ButtonBarTag + ButtonNumber],
         ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag],
         ButtonMan.ButtonHotKeyPos[ButtonBarTag + ButtonNumber]
   EndForeach
   Dynarray HotKeyBag[]
   PackMan.UnpackAnyArrayFrom(
      ButtonMan.Property[ButtonBarTag + "HotKeyBag"],
      HotKeyBag)
   EventMan.ClearHandlersWith(HotKeyBag)
   ForEach Element in HotKeyBag
      Release Vars ButtonMan.HotKeyButtonPtr[Element]
   EndForEach
   ForEach Element in ButtonMan.DefaultPropertyBag
      Release Vars ButtonMan.Property[ButtonBarTag + Element]
   EndForeach
   ButtonMan.MethodBag["ObjectTag"] = ButtonBarTag
   EventMan.ClearHandlersWith(ButtonMan.MethodBag)
   Release Vars ButtonMan.MethodBag["ObjectTag"]
EndProc ; ButtonMan.DestroyButtonBar

Proc ButtonMan.UserDialog()
   Private
      PropertyBag,
      WindowBag,
      ColorBag,
      Element,
      ButtonBarTag,
      PropertyConversion,
      IsShadowColor,
      ShadowColor,
      StartingWindow
   If Not ButtonMan.UserCanConfigure Then
      Return 1
   Endif
   StartingWindow = GetWindow()
   ButtonBarTag = EventMan.WindowObjectTag[EventMan.TargetWindow]
   Window GetAttributes EventMan.TargetWindow To WindowBag
   Window GetColors EventMan.TargetWindow To ColorBag
   Dynarray PropertyBag[]
   Dynarray PropertyConversion[]
   PropertyConversion["Horizontal"] = 1
   PropertyConversion["Vertical"] = 2
   PropertyConversion["Radio"] = 1
   PropertyConversion["Toggle"] = 2
   PropertyConversion["Checkbox"] = 3
   PropertyConversion["Border"] = 1
   PropertyConversion["Shadow"] = 2
   PropertyConversion["Flat"] = 3
   ForEach Element in ButtonMan.DefaultPropertyBag
      PropertyBag[Element] = ButtonMan.Property[ButtonBarTag + Element]
      If IsAssigned(PropertyConversion[PropertyBag[Element]]) Then
         PropertyBag[Element] = PropertyConversion[PropertyBag[Element]]
      Endif
   EndForEach
   ShadowColor = PropertyBag["ShadowColor"]
   IsShadowColor = True
   For Element From 0 to 4
      If ColorBag[Element] <> ShadowColor Then
         IsShadowColor = False
         Quitloop
      Endif
   EndFor
   If ButtonMan.UserDialogBox() Then
      ButtonMan.ApplyDialog()
   Endif
   If IsWindow(StartingWindow) Then
      Window Select StartingWindow
   Endif
   Return 1
EndProc ; ButtonMan.UserDialog

Proc ButtonMan.ApplyDialog()
   Private
      ButtonBag,
      SelectedBag,
      DisabledBag,
      ButtonTag,
      ButtonMan.ApplyingDialog

   ButtonMan.ApplyingDialog = True
   Dynarray ButtonBag[]
   PackMan.UnpackAnyArrayFrom(ButtonMan.Property[ButtonBarTag + "ButtonBag"],
      ButtonBag)
   Dynarray SelectedBag[]
   Dynarray DisabledBag[]
   ForEach ButtonTag in ButtonBag
      Switch
         Case Not ButtonMan.IsButtonEnabled[ButtonBarTag + ButtonTag]:
            DisabledBag[ButtonTag] = True
         Case ButtonMan.IsButtonTrue[ButtonBarTag + ButtonTag]:
            SelectedBag[ButtonTag] = True
      EndSwitch
   EndForeach
   ButtonMan!ConvertNumsToProps()
   ButtonMan.DestroyButtonBar(ButtonBarTag)
   ButtonMan.MakeButtonBar(PropertyBag,ButtonBag,WindowBag,ColorBag)
   Foreach ButtonTag in SelectedBag
      ButtonMan.SelectButton(ButtonBarTag,ButtonTag)
   EndForEach
   ButtonMan.MethodToFire = ""
   ForEach ButtonTag in DisabledBag
      ButtonMan.EnableButton(ButtonBarTag,ButtonTag,False)
   EndForEach
EndProc ; ButtonMan.ApplyDialog

Proc ButtonMan.UserDialogBox()
   Private
      Apply,
      Color,
      SaveDialog,
      DefaultAction,
      ControlVar,
      ReturnValue

   ; PaintPAL_Generated_Code_Begin(440506917)

   SHOWDIALOG "ButtonBar Property Inspector"
      PROC "ButtonMan.UserDialogProc"
         TRIGGER "UPDATE"

      @3,20 HEIGHT 18 WIDTH 42

      ; PaintPAL_Frame_Begin
      FRAME SINGLE FROM 0,18 TO 9,39
      PAINTCANVAS ATTRIBUTE 112 0,18,0,39
      PAINTCANVAS ATTRIBUTE 112 0,18,9,18
      PAINTCANVAS ATTRIBUTE 127 9,19,9,39
      PAINTCANVAS ATTRIBUTE 127 0,39,9,39
      ; PaintPAL_Frame_End

      ; PaintPAL_Frame_Begin
      FRAME SINGLE FROM 0,0 TO 9,17
      PAINTCANVAS ATTRIBUTE 112 0,0,0,17
      PAINTCANVAS ATTRIBUTE 112 0,0,9,0
      PAINTCANVAS ATTRIBUTE 127 9,1,9,17
      PAINTCANVAS ATTRIBUTE 127 0,17,9,17
      ; PaintPAL_Frame_End

      ; PaintPAL_Frame_Begin
      FRAME SINGLE FROM 10,0 TO 15,17
      PAINTCANVAS ATTRIBUTE 112 10,0,10,17
      PAINTCANVAS ATTRIBUTE 112 10,0,15,0
      PAINTCANVAS ATTRIBUTE 127 15,1,15,17
      PAINTCANVAS ATTRIBUTE 127 10,17,15,17
      ; PaintPAL_Frame_End

      RADIOBUTTONS @2,1 HEIGHT 2 WIDTH 16
         "Horizontal",
         "Vertical"
         TAG "Orientation"
         TO PropertyBag["Orientation"]

      LABEL @1,2
         "O~r~ientation"
         FOR "Orientation"

      RADIOBUTTONS @5,1 HEIGHT 3 WIDTH 12
         "Border",
         "Shadow",
         "Flat"
         TAG "ButtonStyle"
         TO PropertyBag["ButtonStyle"]

      LABEL @4,2
         "~B~utton style"
         FOR "ButtonStyle"

      ACCEPT @8,12 WIDTH 5
         "s"
         MIN 1
         MAX 50
         REQUIRED
         TAG "WrapCount"
         TO PropertyBag["WrapCount"]

      LABEL @8,1
         "WrapCount:"
         FOR "WrapCount"

      CHECKBOXES @2,19 HEIGHT 5 WIDTH 15
         TAG "WindowAttributes"
         "HasFrame" TO WindowBag["HasFrame"],
         "CanMove" TO WindowBag["CanMove"],
         "CanResize" TO WindowBag["CanResize"],
         "Floating" TO WindowBag["Floating"],
         "HasShadow" TO WindowBag["HasShadow"]

      LABEL @1,20
         "~W~indow"
         FOR "WindowAttributes"

      ACCEPT @7,30 WIDTH 7
         "s"
         TAG "OriginRow"
         TO WindowBag["OriginRow"]

      LABEL @7,19
         "OriginRow:"
         FOR "OriginRow"

      ACCEPT @8,30 WIDTH 7
         "s"
         TAG "OriginCol"
         TO WindowBag["OriginCol"]

      LABEL @8,19
         "OriginCol:"
         FOR "OriginCol"

      RADIOBUTTONS @12,1 HEIGHT 3 WIDTH 14
         "Radio",
         "Toggle",
         "Checkbox"
         TAG "Action"
         TO PropertyBag["Action"]

      LABEL @11,2
         "Action (info)"
         FOR "Action"

      PUSHBUTTON @10,18 WIDTH 11
         "~C~olor"
         VALUE ButtonMan.Color()
         TAG "Color"
         TO Color

      PUSHBUTTON @10,29 WIDTH 11
         "Pre~v~iew"
         VALUE ButtonMan.Preview()
         TAG "Preview"
         TO Preview

      PUSHBUTTON @12,18 WIDTH 11
         "~S~ave"
         VALUE ButtonMan.SaveDialog()
         TAG "Save"
         TO SaveDialog

      PUSHBUTTON @12,29 WIDTH 11
         "~D~efault"
         VALUE ButtonMan.RestoreDefault()
         TAG "Default"
         TO DefaultAction

      PUSHBUTTON @14,18 WIDTH 11
         "~O~k"
         OK
         DEFAULT
         VALUE "Ok"
         TAG "Ok"
         TO ControlVar

      PUSHBUTTON @14,29 WIDTH 11
         "~C~ancel"
         CANCEL
         VALUE "Cancel"
         TAG "Cancel"
         TO ControlVar
   ENDDIALOG

   ; PaintPAL_Generated_Code_End(440506917)
   ReturnValue = Retval
   ButtonMan.DestroyButtonBar("$Preview$")
   Return ReturnValue
EndProc ; ButtonMan.UserDialogBox

Proc ButtonMan.UserDialogProc(TriggerType,TagValue,EventValue,ElementValue)
   Private
      ReturnVal
   ReturnVal = True
   Switch
      Case TriggerType = "UPDATE":
         Switch
            Case TagValue = "Action":
               If ControlValue("Action") <> PropertyBag["Action"] Then
                  ResyncControl TagValue
                  Beep
                  Message "Cannot change action - information only"
                  ReturnVal = False
               Endif
            Case TagValue = "WindowAttributes":
               If (Not ControlValue(TagValue,"Floating")) And
                  PropertyBag["MustFloat"] Then
                  Beep
                  Message "Floating must be set for this buttonbar"
                  WindowBag["Floating"] = True
                  ResyncControl "WindowAttributes"
                  ReturnVal = False
               Endif
         EndSwitch
   EndSwitch
   Return ReturnVal
EndProc ; ButtonMan.UserDialogProc

Proc ButtonMan.SaveDialog()
   Private
      UserScript,
      Element,
      Command
   If SysMode() <> "Main" Then
      Beep
      Message "Cannot save dialog in this context"
      Return True
   Endif
   UserScript = PropertyBag["UserScript"]
   If IsBlank(UserScript) Then
      Beep
      Message "Cannot save configuration for this buttonbar"
      Return True
   Endif
   ShowPopup "Save custom configuration?" Centered
      "No":"Do NOT save custom configuration file for button bar":"No",
      "Yes":"Save custom configuration file for button bar":"Yes"
   EndMenu
   To Command
   If Command <> "Yes" Then
      Return true
   Endif
   UserScript = Privdir() + UserScript
   ButtonMan!ConvertNumsToProps()
   If IsFile(UserScript + ".sc") Then
      {Tools} {Delete} {Script} Select UserScript {Ok}
   Endif
   UserScript = UserScript + ".sc"
   Message "Saving configuration to ",UserScript
   ForEach Element in ButtonMan.InterfaceProperties
      Print File UserScript
         "PropertyBag[\"" + Element + "\"] = ",
            IIf(Substr(Type(PropertyBag[Element]),1,1) <> "A",
               PropertyBag[Element],"\"" + PropertyBag[Element] + "\""),
               "\n"
   EndForEach
   ForEach Element in ButtonMan.DefaultWindowBag
      Print File UserScript
         "WindowBag[\"" + Element + "\"] = ",
            IIf(Substr(Type(WindowBag[Element]),1,1) <> "A",
               WindowBag[Element],"\"" + WindowBag[Element] + "\""),
               "\n"
   EndForEach
   If Not IsShadowColor Then
      ForEach Element in ButtonMan.DefaultColorBag
         Print File UserScript
            "ColorBag[\"" + Element + "\"] = ",ColorBag[Element],"\n"
      EndForEach
   Endif
   ButtonMan!ConvertPropsToNums()
   Return True
EndProc ; ButtonMan.SaveDialog

Proc ButtonMan.RestoreDefault()
   Private
      UserScript,
      Command
   UserScript = PropertyBag["UserScript"]
   If IsBlank(UserScript) Then
      Beep
      Message "No custom configuration exists to be removed"
      Return True
   Endif
   If SysMode() <> "Main" Then
      Beep
      Message "Cannot restore default in this context"
      Return True
   Endif
   UserScript = Privdir() + UserScript
   If IsFile(UserScript + ".sc") Then
      ShowPopup "Delete custom configuration?" Centered
         "No":"Do NOT delete custom configuration file for button bar":"No",
         "Yes":"Delete custom configuration file for button bar":"Yes"
      EndMenu
      To Command
      If Command = "Yes" Then
         {Tools} {Delete} {Script} Select UserScript {Ok}
         Message
            "ButtonBar will be returned to default config next time it loads"
      Endif
   Else
      Message "No custom configuration file to delete"
   Endif
   Return True
EndProc

Proc ButtonMan.Preview()
   Private
      ButtonBag,
      OriginRow,
      OriginCol,
      ObjectTag,
      Title,
      UserScript,
      ButtonTag,
      ButtonNumber

   Dynarray ButtonBag[]
   ButtonBag["First"] = Chr(1) + ",~S~elected"
   ButtonBag["Second"] = Chr(2) + ",~B~utton"
   ButtonBag["Third"] = Chr(3) + ",~D~isabled"
   OriginRow = WindowBag["OriginRow"]
   OriginCol = WindowBag["OriginCol"]
   Title = WindowBag["Title"]
   ObjectTag = PropertyBag["ObjectTag"]
   UserScript = PropertyBag["UserScript"]
   ButtonTag = PropertyBag["ButtonTag"]
   ButtonNumber = PropertyBag["ButtonNumber"]
   Release Vars
      PropertyBag["ButtonTag"],
      PropertyBag["ButtonNumber"],
      PropertyBag["UserScript"]
   PropertyBag["ObjectTag"] = "$Preview$"
   ButtonMan!ConvertNumsToProps()
   WindowBag["OriginRow"] = 1
   If PropertyBag["Orientation"] = "Horizontal" Then
      WindowBag["OriginCol"] = 42
   Else
      WindowBag["OriginCol"] = 65
   Endif
   WindowBag["Title"] = "Preview"
   ButtonMan.DestroyButtonBar("$Preview$")
   ButtonMan.MakeButtonBar(PropertyBag,ButtonBag,WindowBag,ColorBag)
   ButtonMan.SelectButton("$Preview$","First")
   ButtonMan.EnableButton("$Preview$","Third",False)
   WindowBag["OriginRow"] = OriginRow
   WindowBag["OriginCol"] = OriginCol
   WindowBag["Title"] = Title
   PropertyBag["ObjectTag"] = ObjectTag
   PropertyBag["UserScript"] = UserScript
   PropertyBag["ButtonTag"] = ButtonTag
   PropertyBag["ButtonNumber"] = ButtonNumber
   ButtonMan!ConvertPropsToNums()
   Return True
EndProc ; ButtonMan.Preview

Proc ButtonMan!ConvertPropsToNums()
   Private
      Element
   ForEach Element in ButtonMan.DefaultPropertyBag
      If IsAssigned(PropertyConversion[PropertyBag[Element]]) Then
         PropertyBag[Element] = PropertyConversion[PropertyBag[Element]]
      Endif
   EndForEach
EndProc ; ButtonMan!ConvertPropToNums

Proc ButtonMan!ConvertNumsToProps()
   Private
      Element
   ForEach Element in PropertyBag
      Switch
         Case Element = "ORIENTATION":
            Switch
               Case PropertyBag[Element] = 1:
                  PropertyBag[Element] = "Horizontal"
               Case PropertyBag[Element] = 2:
                  PropertyBag[Element] = "Vertical"
            EndSwitch
         Case Element = "ACTION":
            Switch
               Case PropertyBag[Element] = 1:
                  PropertyBag[Element] = "Radio"
               Case PropertyBag[Element] = 2:
                  PropertyBag[Element] = "Toggle"
               Case PropertyBag[Element] = 3:
                  PropertyBag[Element] = "Checkbox"
            EndSwitch
         Case Element = "BUTTONSTYLE":
            Switch
               Case PropertyBag[Element] = 1:
                  PropertyBag[Element] = "Border"
               Case PropertyBag[Element] = 2:
                  PropertyBag[Element] = "Shadow"
               Case PropertyBag[Element] = 3:
                  PropertyBag[Element] = "Flat"
            EndSwitch
      EndSwitch
   EndForEach
EndProc ; ButtonMan!ConvertNumsToProps

Proc ButtonMan.Color()
   Private
      Palette,
      ControlVar,
      ReturnValue,
      DepartedControl,
      ColorNames,
      CopyColors

   Dynarray ColorNames[]
   ColorNames["CanvasColor"] = "PropertyBag[\"CanvasColor\"]"
   ColorNames["ButtonColor"] = "PropertyBag[\"ButtonColor\"]"
   ColorNames["HotKeyColor"] = "PropertyBag[\"HotKeyColor\"]"
   ColorNames["SelectedColor"] = "PropertyBag[\"SelectedColor\"]"
   ColorNames["SelectedHotKeyColor"] = "PropertyBag[\"SelectedHotKeyColor\"]"
   ColorNames["DisableColor"] = "PropertyBag[\"DisableColor\"]"
   ColorNames["ShadowColor"] = "PropertyBag[\"ShadowColor\"]"
   ColorNames["HighlightColor"] = "PropertyBag[\"HighLightColor\"]"
   ColorNames["InActiveWindow"] = "ColorBag[0]"
   ColorNames["ActiveWindow"] = "ColorBag[1]"
   ColorNames["Icons"] = "ColorBag[2]"
   ColorNames["ScrollBars"] = "ColorBag[3]"
   ColorNames["Controls"] = "ColorBag[4]"

   Dynarray CopyColors[]
   CopyColors["InActiveWindow"] = True
   CopyColors["ActiveWindow"] = True
   CopyColors["Icons"] = True
   CopyColors["ScrollBars"] = True
   CopyColors["Controls"] = True

   ; PaintPAL_Generated_Code_Begin(440506913)

   SHOWDIALOG "ButtonBar Color Box"
      PROC "ButtonMan.ColorProc"
         MOUSE "DOWN"
         KEY "Down", "Left", "Right", "Up"
         TRIGGER "ARRIVE", "UPDATE", "DEPART"

      @7,17 HEIGHT 15 WIDTH 49

      ; PaintPAL_Frame_Begin
      FRAME SINGLE FROM 0,0 TO 10,24
      PAINTCANVAS ATTRIBUTE 112 0,0,0,24
      PAINTCANVAS ATTRIBUTE 112 0,0,10,0
      PAINTCANVAS ATTRIBUTE 127 10,1,10,24
      PAINTCANVAS ATTRIBUTE 127 0,24,10,24
      ; PaintPAL_Frame_End

      ; PaintPAL_Frame_Begin
      FRAME SINGLE FROM 0,25 TO 8,46
      PAINTCANVAS ATTRIBUTE 112 0,25,0,46
      PAINTCANVAS ATTRIBUTE 112 0,25,8,25
      PAINTCANVAS ATTRIBUTE 127 8,26,8,46
      PAINTCANVAS ATTRIBUTE 127 0,46,8,46
      ; PaintPAL_Frame_End

      ; PaintPAL_Static_Text_Begin
      PAINTCANVAS FILL " " ATTRIBUTE 112 1,2,1,15
      @1,2
      ?? "Canvas colors:"
      PAINTCANVAS ATTRIBUTE 112 1,2,1,15
      ; PaintPAL_Static_Text_End

      ; PaintPAL_Static_Text_Begin
      PAINTCANVAS FILL " " ATTRIBUTE 112 1,27,2,42
      @1,27
      ?? "Frame colors:"
      @2,27
      ?? ""
      PAINTCANVAS ATTRIBUTE 112 1,27,2,42
      ; PaintPAL_Static_Text_End

      ACCEPT @2,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "CanvasColor"
         TO PropertyBag["CanvasColor"]

      LABEL @2,1
         "Canvas........."
         FOR "CanvasColor"

      ACCEPT @3,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "ButtonColor"
         TO PropertyBag["ButtonColor"]

      LABEL @3,1
         "Button........."
         FOR "ButtonColor"

      ACCEPT @4,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "HotKeyColor"
         TO PropertyBag["HotKeyColor"]

      LABEL @4,1
         "HotKey........."
         FOR "HotKeyColor"

      ACCEPT @5,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "SelectedColor"
         TO PropertyBag["SelectedColor"]

      LABEL @5,1
         "Selected......."
         FOR "SelectedColor"

      ACCEPT @6,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "SelectedHotKeyColor"
         TO PropertyBag["SelectedHotKeyColor"]

      LABEL @6,1
         "SelectedHotKey."
         FOR "SelectedHotKeyColor"

      ACCEPT @7,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "DisableColor"
         TO PropertyBag["DisableColor"]

      LABEL @7,1
         "Disable........"
         FOR "DisableColor"

      ACCEPT @8,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "ShadowColor"
         TO PropertyBag["ShadowColor"]

      LABEL @8,1
         "Shadow........."
         FOR "ShadowColor"

      ACCEPT @9,17 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "HighlightColor"
         TO PropertyBag["HighlightColor"]

      LABEL @9,1
         "Highlight......"
         FOR "HighlightColor"

      CHECKBOXES @2,26 HEIGHT 1 WIDTH 20
         TAG "IsShadowColor"
         "~S~hadowColor, or" TO IsShadowColor

      ACCEPT @3,38 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "InActiveWindow"
         TO ColorBag[0]

      LABEL @3,26
         "Inactive..."
         FOR "InActiveWindow"

      ACCEPT @4,38 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "ActiveWindow"
         TO ColorBag[1]

      LABEL @4,26
         "Active....."
         FOR "ActiveWindow"

      ACCEPT @5,38 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "Icons"
         TO ColorBag[2]

      LABEL @5,26
         "Icons......"
         FOR "Icons"

      ACCEPT @6,38 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "ScrollBars"
         TO ColorBag[3]

      LABEL @6,26
         "ScrollBars."
         FOR "ScrollBars"

      ACCEPT @7,38 WIDTH 7
         "s"
         MIN 0
         MAX 127
         REQUIRED
         TAG "Controls"
         TO ColorBag[4]

      LABEL @7,26
         "Controls..."
         FOR "Controls"

      PUSHBUTTON @9,26 WIDTH 19
         "~P~alette"
         VALUE ButtonMan.Palette()
         TAG "Palette"
         TO Palette

      PUSHBUTTON @11,4 WIDTH 12
         "Pre~v~iew"
         VALUE ButtonMan.Preview()
         TAG "Preview"
         TO Preview

      PUSHBUTTON @11,18 WIDTH 12
         "~O~k"
         OK
         DEFAULT
         VALUE "Ok"
         TAG "Ok"
         TO ControlVar

      PUSHBUTTON @11,33 WIDTH 12
         "~C~ancel"
         CANCEL
         VALUE "Cancel"
         TAG "Cancel"
         TO ControlVar
   ENDDIALOG

   ; PaintPAL_Generated_Code_End(440506913)
   ReturnValue = Retval
   If IsAssigned(ColorPal.IsActive) Then
      ColorPal.DestroyPalette()
      ColorPal.Destructor()
   Endif
   Return ReturnValue
EndProc ; ButtonMan.Color

Proc ButtonMan.Palette()
   If IsAssigned(ColorPal.IsActive) Then
      ColorPal.DestroyPalette()
      ColorPal.Destructor()
   Else
      ColorPal.Constructor()
      ColorPal.MakePalette()
      Message "For each field, select color on palette with mouse or arrow keys"
   Endif
   Return True
EndProc ; ButtonMan.Palette

Proc ButtonMan.ColorProc(TriggerType,TagValue,EventValue,ElementValue)
   Private
      EventType,
      ReturnVal,
      Element
   Switch
      Case TriggerType = "EVENT":
         EventType = EventValue["Type"]
         Switch
            Case EventType = "KEY":
               If IsAssigned(ColorPal.IsActive) Then
                  If ColorPal.SelectAttribute(EventValue) Then
                     If IsAssigned(ColorNames[TagValue]) Then
                        If Not (IsAssigned(CopyColors[TagValue]) And
                           IsShadowColor) Then
                           Execute ColorNames[TagValue] + " = ColorPal.ColorAttribute"
                           ResyncControl TagValue
                        Endif
                     Endif
                  Endif
                  ReturnVal = False
               Else
                  ReturnVal = True
               Endif
            Case EventType = "MOUSE": ; DOWN only
               If IsAssigned(ColorPal.IsActive) And
                  WindowAt(EventValue["Row"],EventValue["Col"]) =
                     ColorPal.WindowHandle Then
                  If ColorPal.SelectAttribute(EventValue) Then
                     If IsAssigned(ColorNames[TagValue]) Then
                        If Not (IsAssigned(CopyColors[TagValue]) And
                           IsShadowColor) Then
                           Execute ColorNames[TagValue] +
                              " = ColorPal.ColorAttribute"
                           ResyncControl TagValue
                        Endif
                     Endif
                  Endif
                  EventMan.ReturnFromMouseMode()
                  ReturnVal = False
               Else
                  ReturnVal = True
               Endif
         EndSwitch
      Case TriggerType = "DEPART":
         If IsAssigned(ColorNames[TagValue]) Then
            DepartedControl = TagValue ; for update Palette
         Endif
         ReturnVal = True
      Case TriggerType = "UPDATE":
         Switch
            Case TagValue = "Palette":
               ResyncControl TagValue
               If DepartedControl <> "" Then
                  SelectControl DepartedControl
                  If IsAssigned(ColorPal.IsActive) And
                     IsAssigned(ColorNames[DepartedControl]) Then
                     ColorPal.ShowAttribute(ControlValue(DepartedControl))
                  Endif
                  DepartedControl = ""
               Endif
               ReturnVal = False
            Case TagValue = "Preview":
               ResyncControl TagValue
               If DepartedControl <> "" Then
                  SelectControl DepartedControl
                  DepartedControl = ""
               Endif
               ReturnVal = False
            Case IsAssigned(ColorNames[TagValue]):
               If Not (IsAssigned(CopyColors[TagValue]) And
                  IsShadowColor) Then
                  If IsAssigned(ColorPal.IsActive) Then
                     ColorPal.ShowAttribute(ControlValue(TagValue))
                  Endif
                  If (IsShadowColor And TagValue = "ShadowColor") Then
                     ForEach Element In CopyColors
                        Execute ColorNames[Element] +
                           " = ControlValue(TagValue)"
                        ResyncControl Element
                     EndForeach
                  Endif
                  ReturnVal = True
               Else
                  If ControlValue(TagValue) <> PropertyBag["ShadowColor"] Then
                     ResyncControl TagValue
                     Beep
                     Message "Cannot change window color when ShadowColor is set"
                     ReturnVal = False
                  Else
                     ReturnVal = True
                  Endif
               Endif
            Case TagValue = "IsShadowColor":
               If ControlValue(TagValue,"~S~hadowColor, or") Then
                  ForEach Element In CopyColors
                     Execute ColorNames[Element] +
                        " = ControlValue(\"ShadowColor\")"
                     ResyncControl Element
                  EndForeach
               Endif
               ReturnVal = True
            Otherwise:
               ReturnVal = True
         EndSwitch
      Case TriggerType = "ARRIVE":
         If IsAssigned(ColorPal.IsActive) And
            IsAssigned(ColorNames[TagValue]) Then
            ColorPal.ShowAttribute(ControlValue(TagValue))
         Endif
         ReturnVal = True
   EndSwitch
   Return ReturnVal
EndProc ; ButtonMan.ColorProc